#include <math.h>

#define AK  0.0174533

void plteu(double O[], double E1, double E2, double E3)

/*       plteu.c
   [Euler angles]
   Angulos de Euler(E1,E2,E3).
   O = R1 * R2 * R3.
   R1 y R3 son rotaciones en sentido contrario de las manecillas del reloj
   en el plano X-Y a traves de los angulos E1 y E3 respectivamente.
   R2 es una rotacion en el plano Y-Z atravez del angulo E2 en sentido
   contrario de las manecillas del reloj.

   O[3][3]   Matriz de rotacion.
   E1,E2,E3  Angulos de Euler (grados de rotacion).
   (H.V.McIntosh, 30.MAY.75)
   (Fco. Gil, 20.ENE.85)
   (Conv. a C, G.Cisneros, 19.JUL.88)
   (Angel Canto, 20.Feb.89)
*/
{
 double ee1,ee2,ee3,c1,c2,c3,s1,s2,s3;

 ee1 = E1 * AK;
 ee2 = E2 * AK;
 ee3 = E3 * AK;
 c1 = cos(ee1);
 c2 = cos(ee2);
 c3 = cos(ee3);
 s1 = sin(ee1);
 s2 = sin(ee2);
 s3 = sin(ee3);
 O[0] = c1*c3-s1*c2*s3;  /* O[0][0] = O(1,1).    */
 O[1] = -c1*s3-s1*c2*c3; /* O[0][1] = O(1,2).    */
 O[2] = s1*s2;           /* O[0][2] = O(1,3).    */
 O[3] = s1*c3+c1*c2*s3;  /* O[1][0] = O(2,1).    */
 O[4] = -s1*s3+c1*c2*c3; /* O[1][1] = O(2,2).    */
 O[5] = -c1*s2;          /* O[1][2] = O(2,3).    */
 O[6] = s2*s3;           /* O[2][0] = O(3,1).    */
 O[7] = s2*c3;           /* O[2][1] = O(3,2).    */
 O[8] = c2;              /* O[2][2] = O(3,3).    */
}
