#include "kon.h"

void pltor(double Z1, double ZE[], double Z2, int NZ, int KX, int NX,
	   int KY, int NY, void (*graff)())

/*       pltor.c
   [Orthogonal relief]
   Relieve ortogonal.
   Toma los valores de la funcion de un arreglo.
   ZE[NX][NY].
   (Z1,Z2) Rango donde se varia el nivel del contorno.
   ZE[]    Arreglo con los valores de la funcion.
   NZ	   Numero de niveles de contornos.
   KX      Numero de bloques en la dir. X.
   NX      Numero de puntos en la dir X.
   KY      Numero de bloques en la dir. Y.
   NY      Numero de puntos en la dir Y.
   graff   Funcion de conversion de coordenadas.
   (H.V.McIntosh, 20.NOV.74)
   (Fco. Gil, 17.ENE.85)
   (Conv. a C, G.Cisneros, 19.JUL.88)
   (Angel Canto, 20.Feb.89)
*/
{
  int i0,j0,ix,iy,ii,jj,ja,jb,ia,ib,k;
  double zz,xf,yf,dz,z;

 if(KX == 0 || KY == 0 || NX < 2 || NY < 2 || NZ < 2) return;

 i0 = konmx((NX / (2*KX)), 1) + 1;
 j0 = konmx((NY / (2*KY)), 1) + 1;
 ix = 2 * i0;
 iy = 2 * j0;
 zz = Z2 - Z1;
 xf = zz/(double)(NX-1);
 yf = zz/(double)(NY-1);
 dz = (3.0*zz)/(double)(NZ-1);

 ii = i0 + 1;
 jj = j0 + 1;
 do
  {
    ja = konmx((jj-j0),1);
    jb = konmn((jj+j0),NY);
    do
     {
       ia = konmx((ii-i0),1);
       ib = konmn((ii+i0),NX);

       z = Z1 - zz;
       for (k=0 ; k < NZ ; k++)
	{  konoc(z, -xf, yf, ia, ib, ja, jb, ZE, NX, NY, graff);
           z += dz;};
       ii += ix;
     }  while (((ii - i0) < NX) && ((ii + i0) > 1));
    ix = -ix;
    ii += ix;
    jj += iy; }  while (NY > (jj - j0));
}
