#include <stdio.h>
#include <ctype.h>
#include <math.h>


extern void pltfm();
extern void pltca();

#define UP      0
#define DOWN    1

#define ex(x)   ((x-X1) * SCX)
#define wy(y)   ((y-Y1) * SCY)

#define n       101     /*      Numero de puntos.                       */
#define dt      0.01    /*      Incremento.                             */

static double  X1  = 0.0;       /* Guarda Cotas.                        */
static double  X2  = 1.0;
static double  Y1  = 0.0;
static double  Y2  = 1.0;
static double  SCX = 1.0;
static double  SCY = 1.0;


void pltig(double x, double y, int l, void (*graff)())

/*       pltig.c
   [INCREMENTAL GRAPH]
   GRAFICA EN INCREMENTOS.
   l = 1  (X1,Y1) respectivas cotas inferiores.
   l = 2  (X2,Y2) respectivas cotas superiores.
   l = 3  Punto inicial.
   l = 4  Puntos subsecuentes.
   l = 5  Ejes rectangulares a traves de (X,Y).
   l = 6  Marca chica a traves de (X,Y).
   l = 7  Marca grande a traves de (X,Y).
   l = 8  Ejes rectangulares atravez de X e Y con etiquetas X e Y
   l = 9  Ejes rectangulares atravez de X e I con etiquetas X e I
   l = 10 Ejes rectangulares atravez de X e Y con etiquetas X e Y.
	  (Y invertida)
   l = 11 Ejes rectangulares atravez de X e I con etiquetas X e I.
	  (I invertida)
   (H.V.McIntosh, 6.JUN.75)
   (Fco. Gil, 6.OCT.85) [F80].
   (Conv. a C, G.Cisneros, 29.JUN.88)
   (Angel Canto, 21.Ene.89)
   (J.P.H.E., Nov.93)
*/
{ char   bnf[13];
  int    i, j, k;
  double tt,rr,de,xt,tdt, xp, yp;

  if ((l == 3) || (l > 4))
  {
    if (X2 != X1) SCX = 1.0/(X2-X1);
    if (Y2 != Y1) SCY = 1.0/(Y2-Y1); };

  switch (l) {
    case 1: X1 = x ; Y1 = y ; return;
    case 2: X2 = x ; Y2 = y ; return;
    case 3: (*graff)(ex(x), wy(y), UP); return;
    case 4: (*graff)(ex(x), wy(y), DOWN); return;
    case 5: x = ex(x) ; y = wy(y);
	    tt = 0.0; (*graff)(tt, y, UP);
	    for (i=1 ; i < n ; i++)
	    { tt += dt ; (*graff)(tt, y, DOWN);};
	    tt = 0.0; (*graff)(x, tt, UP);
	    for (i=1 ; i < n ; i++)
	    { tt += dt ; (*graff)(x, tt, DOWN);}; return;
    case 6: rr = 0.015 ; pltfm(ex(x), wy(y), rr, graff); return;
    case 7: rr = 0.020 ; pltfm(ex(x), wy(y), rr, graff); return;
    case 8:
    case 9: xp = x;      yp = y;
	    x  = ex(x);  y  = wy(y);
	    tt = 0.0; (*graff)(tt, y, UP);
	    for (i=1 ; i < n ; i++)
	    { tt += dt ; (*graff)(tt, y, DOWN);}

	    tt=0.0;  de=fabs(x/5.0);  xt=X1;  tdt=fabs((xp-X1)/5.0);
	    if( de > 0.05 ) {
		for (i=0 ; i < 4 ; i++)
		{ tt += de; xt += tdt;  pltfm(tt, y, 0.015, graff);
		  sprintf(bnf,"%10.1f", xt);
		  for(j=0, k=0; j < 10; j++, k++) {
		      while(bnf[j]==0x20) j++; bnf[k]=bnf[j];  }
		  plttp(tt-0.04, y-0.04, -0.017, bnf, 0.0, 5, 1, pltca); }
	    }

	    tt=x;    de=fabs((1.0-x)/5.0),  xt=xp; tdt=fabs((X2-xp)/5.0);
	    if( de > 0.05 ) {
		for (i=0 ; i < 4 ; i++)
		{ tt += de; xt += tdt;  pltfm(tt, y, 0.015, graff);
		  sprintf(bnf,"%10.1f", xt);
		  for(j=0, k=0; j < 10; j++, k++) {
		      while(bnf[j]==0x20) j++; bnf[k]=bnf[j];  }
		  plttp(tt-0.03, y-0.04, -0.017, bnf, 0.0, 5, 1, pltca); }
	    }
	    plttp(0.96, y+0.04, -0.02, "X", 0.0, 1, 1, pltca);


	    tt = 0.0; (*graff)(x, tt, UP);
	    for (i=1 ; i < n ; i++)
	    { tt += dt ; (*graff)(x, tt, DOWN);}

	    tt=0.0;  de = y/5.0;  xt=Y1;  tdt=fabs((yp-Y1)/5.0);
	    if( de > 0.04 ) {
		for(i=0 ; i < 4 ; i++)
		    { tt += de; xt += tdt; pltfm(x, tt, 0.015, graff);
		      sprintf(bnf,"%10.1f", xt);
		      for(j=0, k=0; j < 10; j++, k++) {
			  while(bnf[j]==0x20) j++; bnf[k]=bnf[j];  }
		      plttp(x+0.04, tt, -0.017, bnf, 0.0, 5, 1, pltca); }
	    }

	    tt=y;    de=fabs((1.0-y)/5.0);  xt=yp;  tdt=fabs((Y2-yp)/5.0);
	    if( de > 0.04 ) {
		for(i=0 ; i < 4 ; i++)
		    { tt += de; xt += tdt; pltfm(x, tt, 0.015, graff);
		      sprintf(bnf,"%10.1f", xt);
		      for(j=0, k=0; j < 10; j++, k++) {
			  while(bnf[j]==0x20) j++; bnf[k]=bnf[j];  }
		      plttp(x+0.04, tt, -0.017, bnf, 0.0, 5, 1, pltca); }
	    }

	    if(l==8) plttp(x+0.04, 0.98, -0.02, "Y", 0.0, 1, 1, pltca);
	    if(l==9) plttp(x+0.04, 0.98, -0.02, "I", 0.0, 1, 1, pltca);
	    return;

    case 10:
    case 11: xp = x;     yp = y;
	     x  = ex(x);  y = wy(y);

	    tt = 0.0; (*graff)(tt, 1.0-y, UP);
	    for (i=1 ; i < n ; i++)
	    { tt += dt ; (*graff)(tt, 1.0-y, DOWN);}

	    tt=0.0;  de=fabs(x/5.0); xt=X1; tdt=fabs((xp-X1)/5.0);

	    if(de > 0.05) {
	       for(i=0 ; i < 4 ; i++)
		   { tt += de;  xt += tdt;  pltfm(tt, 1.0-y, 0.015, graff);
		   sprintf(bnf,"%10.1f", xt);
		   for(j=0, k=0; j < 10; j++, k++) {
		       while(bnf[j]==0x20) j++; bnf[k]=bnf[j];  }
		   plttp(tt-0.04, 1.0-y-0.04, -0.017, bnf, 0.0, 5, 1, pltca); }
	    }

	    tt=x;    de=fabs((1.0-x)/5.0);  xt=xp; tdt=fabs((X2-xp)/5.0);
	    if( de > 0.05 ) {
		for(i=0 ; i < 4 ; i++)
		    { tt += de;  xt += tdt;  pltfm(tt, 1.0-y, 0.015, graff);
		    sprintf(bnf,"%10.1f", xt);
		    for(j=0, k=0; j < 10; j++, k++) {
			while(bnf[j]==0x20) j++; bnf[k]=bnf[j];  }
		    plttp(tt-0.03, 1.0-y-0.04, -0.017, bnf, 0.0, 5, 1, pltca); }
	    }
	    plttp(0.97, 1.0-y-0.04, -0.02, "X", 0.0, 1, 1, pltca);


	    tt = 0.0; (*graff)(x, tt, UP);
	    for (i=1 ; i < n ; i++)
	    { tt += dt ; (*graff)(x, tt, DOWN);}

	    tt=0.0;  de=fabs((1-y)/5.0); xt=Y2;  tdt=fabs((Y2-yp)/5.0);
	    if( de > 0.04) {
		for(i=0; i < 4; i++)
		    { tt += de;  xt -= tdt; pltfm(x, tt, 0.015, graff);
		    sprintf(bnf,"%10.1f", xt);
		    for(j=0, k=0; j < 10; j++, k++) {
			while(bnf[j]==0x20) j++; bnf[k]=bnf[j];  }
		    plttp(x+0.04, tt, -0.017, bnf, 0.0, 5, 1, pltca); }
	    }

	    tt=1-y;    de=fabs(y/5.0);  xt=yp;  tdt=fabs((yp-Y1)/5.0);
	    if(de > 0.04) {
	    for(i=0; i < 4; i++)
		{ tt += de; xt -= tdt;  pltfm(x, tt, 0.015, graff);
		   sprintf(bnf,"%10.1f", xt);
		   for(j=0, k=0; j < 10; j++, k++) {
		       while(bnf[j]==0x20) j++; bnf[k]=bnf[j];  }
		   plttp(x+0.04, tt, -0.017, bnf, 0.0, 5, 1, pltca); }
	    }
	    if(l==10) plttp(x+0.04, 0.02, -0.02, "Y", 0.0, 1, 1, pltca);
	    if(l==11) plttp(x+0.04, 0.02, -0.02, "I", 0.0, 1, 1, pltca);
	    return;

    default: return;
  };
}

