#include "plt.h"

#define fx(x) xs*(x-xa)
#define wy(y) ys*(y-ya)


void pltur(double xa, double x1, double dx, double x2, double xb, double ya,
	   double y1, double dy, double y2, double yb, double w, (*pl)())

/*       pltur.c
  [Unit reticle]    (Cuadricula unitaria)
  cover the plotter page with a net of fiducial marks indicating
  unit intervals of data.
  (xa,xb)  x-values at x-margins
  (x1,x2)  x-interval to be reticled
  dx       x-distance between centers of fiducial marks
  (ya,yb)  y-values at y-margins
  (y1,y2)  y-interval to be reticled.
  dy       y-distance between centers of fiducial marks.
  w        width of fiducial mark.
  pl       pen movement subroutine
  dx and dy may be signed or may be absolute values, likewise
  the x-, and y-intervals may be either increasing or decreasing.
  pl=pltca is a suitable argument.
  [05-jan-75]
*/
{

   double xs,ys,d,e,s,t,s1,s2,t1,x,y;

   xs = 1.0/(xb-xa);
   ys = 1.0/(yb-ya);
   d = ((xb-xa) >= 0.0) ? fabs(dx) : -fabs(dx);
   e = ((yb-ya) >= 0.0) ? fabs(dy) : -fabs(dy);
   s = (d >= 0.0) ? 1.0 : -1.0;
   t = (e >= 0.0) ? 1.0 : -1.0;
   s1 = s * x1;
   s2 = s * x2;
   t1 = t * y1;
   x = x2;
   y = y2;

   for (;;)
   {
     pltfm(ex(x), wy(y), w, pl);
     x -= d;
     if ((s*x) < s1) || ((s*x) > s2)
     {
       d = -d;
       x -= d;
       y -= e;
       if ((t*y) lt t1) break;
     }
   }
}
