#include <math.h>

#include "vis.h"

#define SIZE 150

void pvids(double Z1, double ZE[], double Z2, int J1, int J2, int MX,
	   int I1, int I2, int MY, double US, double VS, int L, int M,
	   double S, void (*pl)())

/*       pvids.c
   [Diagonal sequence]
   Secuencia Diagonal para colores, Positiva o negativa distinto color.
   ABS(F(x,y)) es la funcion de la grafica.

   (Z1,Z2) Rango de los valores de ZE.
   ZE[]    Arreglo de los valores de la funcion.
   J1,J2   Rango de X (coordenada horizontal).
   MX,MY   ZE[MX][MY].
   I1,I2   Rango de Y (Coordenada de profundidad).
   US,VS   Cortes totales en direcc. U y V (Max. 1.0).
   L       Direcc. de vista e incremento (+:Oeste, -:Este).
   M       Direcc. de vista e incremento (+:Sur, -:Norte).
   S       =1.0 Grafica parte positiva, = -1.0 Parte negativa.
   pl      Funcion de conversion de coordenadas. (usual pltca)
   (H.V.McIntosh, 19.MAY.75)
   (Fco. Gil, 23.DIC.84)
   (Conv. a C, G. Cisneros, 15.JUL.88)
   (Angel Canto, 16.Mar.89)
*/
{
  int P[SIZE];
  double  U[SIZE],V[SIZE];
  int nl,nm,n,il,iu,mm,nn,i0,j0,k0,i,j,k,ix,kk,iv0;
  double te,zs,dui,eui,duj,euj,dvi,evi,eu,ve;

 if ((Z2 == Z1) || (MX < 2) || (MY < 2)) return;
 nl = sgn(L);
 nm = sgn(M);
 n = nl * nm;
 il = I1 - abs(M);
 iu = I2 + abs(M);
 mm = L * MY;
 nn = 1;
 te = 0.5*(double)(nl+1);
 zs = (1.0-VS) / (Z2-Z1);
 dui = -((double)(nl)*US) / (double)(MY-1);
 eui = dui * (double)(M);
 duj = (1.0-US) / (double)(MX-1);
 euj = duj * (double)(L);
 dvi = VS / (double)(MY-1);
 evi = dvi * (double)(M);

 i0 = (nm > 0) ? (I1+M) : (I2+M);
 j0 = (nl > 0) ? J1 : J2;
 k0 = (n > 0) ? -1 : SIZE;


 do{
    do{
	k = k0;
	i = visbd(i0, iu, il);
	j = j0;
	ix = (i-1) + (j-1)*MY;
	eu = te*US + dui*(double)(i-1) + duj*(double)(j-1);
	ve = dvi*(double)(i-1);

	do{
	   if ((i >= I1) && (i <= I2))
	   {
	     k = visbd(k+n, SIZE-1, 0);
	     U[k] = (double)(eu);
	     V[k] = (double)(ve+zs*(fabs(ZE[ix])-Z1));
	     P[k] = (int)((S*ZE[ix] >= 0.0));   };

	     i -= M;
	     ix -= M;
	     eu -= eui;
	     ve -= evi;
	     if ((i < I1) || (i > I2)) break;
	     U[k] = (double)(eu);
	     V[k] = (double)(ve + zs*(fabs(ZE[ix])-Z1));
	     P[k] = (int)((S*ZE[ix] >= 0.0));
	       j += L;
	      ix += mm;
	      eu += euj;
	  } while ((j >= J1) && (j <= I2));

	kk = (int)(((k+2-n*k)/2) -1);
	iv0 = (int)((SIZE+1-n*(SIZE-2*(k+1)+1))/2);
      visch (&U[kk], &V[kk], &P[kk], iv0, nn, pl);
	nn = -nn;
	i0 += M;
      } while ((i0 >=  il) && (i0 <= iu));
  j0 += L;
   } while ((j0 >= J1) && (j0 <= J2));
}
