#include <math.h>

#include "vis.h"

#define SIZE 150
#define M    1

static  double  G1[SIZE],H1[SIZE];
static  double  X1[SIZE],B1[SIZE],T1[SIZE];
static  double  X2[SIZE],B2[SIZE],T2[SIZE];
static  double  U1[SIZE],F1[SIZE];
static  double  U2[SIZE],F2[SIZE];
static  double  U[SIZE],V1[SIZE],V2[SIZE];


void visdo(double Z1, double S1[], double S2[], double Z2, int NX,
	   int MX, int NY, int MY, double US, double VS, double VD,
	   int L, int IS, void (*pl)())

/*       visdo.c
   [Double surface]
   Doble superficie.

   (Z1,Z2)     Rango de los valores de las superficies.
   S1[],S2[]   Arreglo de datos de dos superficies.
   NX,MX       Dimensionamiento comun de los arreglos S1,S2.[MX,MY].
   NY,MY       Seccion usada en S1,S2.[NX,NY]
   US,VS       Total direccion en U,V.
   VD          Direccionamiento vertical en el segmento de la funcion.
   L           = 1 Oeste , = -1 Este.
   IS          Separadas las superficies :
	       = 1 => SI  , = -1 => NO.
   pl          Funcion de conversion de coordenadas.
   (H.V.McIntosh, 15.MAY.75)
   (Fco. Gil, 16.ENE.85)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16.Mar.89)
*/
{
  int    sequ,n,i0,j0,k0,i,j,k,ixji,nk,ii,kk,n1,n2,nu1,nu2,ng1;
  double dd1,ef,el,em,zs,te,dui,duj,dvi,eu,ve;

 n1 = 0;
 n2 = 0;
 sequ = 1;
 n = L * M;
 dd1 = (VD + VS);
 ef = 1.0 - dd1;
 el = (double)(L);
 em = (double)(M);
 zs = ef/(Z2-Z1);
 te = 0.5 * (el+1.0);
 dui = -(el*US) / (double)(MY-1);
 duj = (1.0-US) / (double)(MX-1);
 dvi = VS / (double)(MY-1);
 
 i0 = (M > 0) ? 2 : (NY-1);
 j0 = (L > 0) ? 1 : NX;
 k0 = (n > 0) ? -1 : SIZE;

 do{
    do{
	k = k0;
	i = visbd(i0, NY+1, 0);
	j = j0;
	eu = te*US + dui*(double)(i-1) + duj*(double)(j-1);
	ve = dvi * (double)(i-1);

       do{
	  if((i >= 1) && (i <= NY))
	  {
	    k = visbd(k+n, SIZE-1, 0);
	    U[k] = (double)(eu);
	    ixji = (i-1) + (MY*(j-1));
	    V1[k] = (double)(ve + zs*(S1[ixji]-Z1));
	    V2[k] = (double)(ve + zs*(S2[ixji]-Z1));
	  };
	  i -= M;
	  eu -= dui;
	  ve -= em * dvi;
	  if ((i < 1) || (i > NY)) break;
	  k = visbd(k+n, SIZE-1, 0);
	  U[k] = (double)(eu);
	  ixji = (i-1) + (MY*(j-1));
	  V1[k] = (double)(ve + zs*(S1[ixji]-Z1));
	  V2[k] = (double)(ve + zs*(S2[ixji]-Z1));
	  j += L;
	  eu += el * duj;
	 } while ((j >= 1) && (j <= NX));

	if (L <= 0)
	{
	  nk = SIZE - k;
	  for (kk=0 ; kk < nk ; kk++)
	  {
	    ii = k + kk;
	    U[kk] = U[ii];
	    V1[kk] = V1[ii];
	    V2[kk] = V1[ii];
	  };
	  k = nk-1;     };

	visrb(U2, F2, &nu2, SIZE, U, V1, k+1, U, V2, k+1, 1.0);
	visrb(U1, F1, &nu1, SIZE, U, V1, k+1, U, V2, k+1,-1.0);

	if (IS >= 0)
	{
	  for (kk=0; kk < nu1 ; kk++)
	   F1[kk] = (double)(ef*F1[kk]);
	  for (kk=0; kk < nu2 ; kk++)
	   F2[kk] = (double)(ef*F2[kk]+dd1);    };
	if (!sequ)
	 { visrb(G1, H1, &ng1, SIZE, U2, F2, nu2, X1, B1, n1, 1.0);
	   vishh(X2, B2, T2, &n2, G1, H1, ng1,-1, pl);
	   visrb(G1, H1, &ng1, SIZE, U1, F1, nu1, X2, T2, n2,-1.0);
	   vishh(X1, B1, T1, &n1, G1, H1, ng1, 1, pl);
	   sequ = !sequ; }
	else
	 { visrb(G1, H1, &ng1, SIZE, U1, F1, nu1, X2, T2, n2,-1.0);
	   vishh(X1, B1, T1, &n1, G1, H1, ng1,-1, pl);
	   visrb(G1, H1, &ng1, SIZE, U2, F2, nu2, X1, B1, n1, 1.0);
	   vishh(X2, B2, T2, &n2, G1, H1, ng1, 1, pl);
	   sequ = !sequ; };

	i0 += M;
      } while ((i0 >= 0) && (i0 <= NY+1));
	j0 += L;
   } while ((j0 >= 1) && (j0 <= NX));
}
