#include "vis.h"

#define SIZE 150

static double  U[SIZE], V[SIZE];

void visds(double Z1, double ZE[], double Z2, int J1, int J2, int MX,
	   int I1, int I2, int MY, double US, double VS, int L, int M,
	   void (*pl)())

/*       visds.c
   [Diagonal sequence]
   Secuencia Diagonal.

   (Z1,Z2)   Rango de los valores de Z.
   ZE[]      Arreglo de los valores de la funcion.
   J1,J2     Intervalo de X en la matriz: ZE(I1,J1) --> ZE(I2,J2).
   MX,MY     Dimension ZE[MX][MY].
   I1,I2     Intervalo en Y en la matriz: ZE(I1,J1) --> ZE(I2,J2).
   US,VS     Inclinacion de vista en direccion U y V (MAX = 1.0).
   L         Direccion de vista (+:OESTE , -:ESTE).
   M         Direccion de vista (+:SUR   , -:NORTE).
   pl        Funcion de conversion de coordenadas.
   (H.V.McIntosh, 10.MAY.84)
   (Fco. Gil, 23.DIC.84)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16.Mar.89)
*/
{
  int    nl,nm,n,il,iu,nn,i0,j0,k0,i,j,k,ix,kk,mm;
  double te,zs,dui,eui,duj,euj,dvi,evi,eu,ve;

 if ( (Z2 == Z1) || (MX < 2) || (MY < 2) ) return;
 nl = sgn(L);
 nm = sgn(M);
 n = nl * nm;
 il = I1 - abs(M);
 iu = I2 + abs(M);
 mm = L * MY;
 nn = 1;
 te = (nl > 0) ? 1.0 : 0.0 ;
 zs = (1.0-VS) / (Z2-Z1);
 dui = -((double)(nl)*US) / (double)(MY-1);
 eui = dui*(double)(M);
 duj = (1.0-US) / (double)(MX-1);
 euj = duj*(double)(L);
 dvi = VS / (double)(MY-1);
 evi = dvi * (double)(M);
 i0 = ((nm > 0) ? I1 : I2) + M;
 j0 = ((nl > 0) ? J1 : J2);
 k0 = ((n  > 0) ? -1 : SIZE);

 do {
    do {
	k = k0;
	i = visbd(i0, iu, il);
	j = j0;
	ix = (i-1) + ((j-1)*MY);
	eu = te*US + dui*(double)(i-1) + duj*(double)(j-1);
	ve = dvi*(double)(i-1);

	do { if((i >= I1) && (i <= I2))
	     { k    = visbd(k+n, SIZE-1, 0);
	       U[k] = (double)(eu);
	       V[k] = (double)(ve + zs*(ZE[ix]-Z1));};
	     i   -= M;
	     ix  -= M;
	     eu  -= eui;
	     ve  -= evi;
	     if((i < I1) || (i > I2)) break;
	     k    = visbd(k+n, SIZE-1, 0);
	     U[k] = (double)(eu);
	     V[k] = (double)(ve + zs*(ZE[ix]-Z1));
	     j   += L;
	     ix  += mm;
	     eu  += euj;
	   } while ((j >= J1) && (j <= J2));

	kk = (n > 0) ? 0 : k;
	visho(&U[kk], &V[kk], (int)((SIZE+1-n*(SIZE-2*(k+1)+1))/2),
	      nn, pl);
	nn = -nn;
	i0 += M;
       } while ((i0 >= il) && (i0 <= iu));
	j0 += L;
    } while ((j0 >= J1) && (j0 <= J2));
}
