#include "vis.h"

#define SIZE 150

static double  U[SIZE],V[SIZE];

void visis(double Z1, double ZE[], double Z2, int J1, int J2,
	   int MX, int I1, int I2, int MY, double O[], void (*pl)())

/*       visis.c
   [Inclined sequence]
   Secuencia inclinada.

   (Z1,Z2)   Rango de ZE.
   ZE[]      Arreglo de valores de la funcion.
   J1,J2     ZE(I1,J1) --> ZE(I2,J2).
   MX,MY     Dimension ZE[MX][MY].
   I1,I2     ZE(I1,J1) --> ZE(I2,J2).
   O[]       Matriz ortogonal para rotacion.
   pl        Funcion de conversion de coordenadas.
   (H.V.McIntosh, 30.MAY.75)
   (Fco. Gil, 23.DIC.84)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16.Mar.89)
*/
{
  int l,m,nl,nm,n,il,iu,nn,i0,j0,k0,i,j,k,ix,kk,mm;
  double zs,duj,euj,dvi,evi,eu,ve,uu;
  /*  double te;  */


 l = -(int)((O[3] >= 0.0) ? 1.0 : -1.0);  /* O(2,1) = O[1][0] = O[3] */
 m =  (int)((O[0] >= 0.0) ? 1.0 : -1.0);  /* O(1,1) = O[0][0] = O[0] */
 nl = sgn(l);
 nm = sgn(m);
 n = nl * nm;
 il = I1 - abs(m);
 iu = I2 + abs(m);
 mm = l * MY;
 nn = 1;
 /* te = 0.5*(double)(nl+1); Nunca lo utiliza PLOT84 */
 zs = 1.0 / (Z2-Z1);
 duj = 1.0 / (double)(MX-1);
 euj = duj * (double)(l);
 dvi = 1.0 / (double)(MY-1);
 evi = dvi * (double)(m);

 i0 = (nm > 0) ? (I1+m) : (I2+m);
 j0 = (nl > 0) ? J1 : J2;
 k0 = (nl > 0) ? -1 : SIZE;

 do{
    do{
	k = k0;
	i = visbd(i0, iu, il);
	j = j0;
	ix = (i-1) + ((j-1)*MY);
	eu = duj * (double)(j-1) - 0.5;
	ve = dvi * (double)(i-1) - 0.5;

	do{
	    if((i >= I1) && (i <= I2))
	    {
	      k = visbd(k+n, SIZE-1, 0);
	      uu = zs*(ZE[ix]-Z1);
	      U[k] = (double)(0.667 * (O[0]*eu+O[3]*ve+O[6]*uu) + 0.5);
	      V[k] = (double)(0.667 * (O[1]*eu+O[4]*ve+O[7]*uu) + 0.5);
	    };
	    i -= m;
	    ix -= m;
	    ve -= evi;
	    if((i < I1) || (i > I2)) break;
	    k  = visbd(k+n, SIZE-1, 0);
	    uu = zs*(ZE[ix]-Z1);
	    U[k] = (double)(0.667 * (O[0]*eu+O[3]*ve+O[6]*uu) + 0.5);
	    V[k] = (double)(0.667 * (O[1]*eu+O[4]*ve+O[7]*uu) + 0.5);
	    j += l;
	    ix += mm;
	    eu += euj;
	  } while ((j >= J1) && (j <= J2));

	kk = ((k+2-n*k)/2) - 1;
	visho(&U[kk], &V[kk], (int)((SIZE+1-n*(SIZE-2*(k+1)+1))/2),
	      nn, pl);
	nn = -nn;
	i0 += m;
      } while ((i0 >= il) && (i0 <= iu));
	j0 += l;
   } while ((j0 >= J1) && (j0 <= J2));
}
