#include <math.h>
#include "vis.h"

#define EP1 1.0E-5
#define EP2 0.5E-4

void visrb(double X[], double Y[], int *J, int M, double X1[], 
	   double Y1[], int N1, double X2[], double Y2[], int N2, 
	   double S)

/*       visrb.c
   [Restricted bound]
   Cota restringida.
   El borde es restringido a la primera funcion definida.
   esta funcion es usada para multiples superficies interceptadas.

   *J,M       Dimension actual, maxima dimension de x,y.
   X[],Y[]    Arreglo de los bordes de la funcion.
   N1,N2      Dimesion X1[N1],Y1[N1],X2[N2],Y2[N2].
   X1[],Y1[]  Primera funcion.
   X2[],Y2[]  Segunda funcion.
   S          Tipo de borde:
	      = 1.0 Superior,  =-1.0 Inferior.
   (H.V.McIntosh, 15.MAY.74)
   (Fco. Gil, 9.ENE.85)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16 Mar.89)
*/
{
  int l,j1,j2;
  double z1,z2,z,w1,w2,w,d,e,u,ww;
  double s1,s2;

 e = u = ww = 0.0;
 l = 1;
 *J = -1;
 j1 = 0;
 j2 = 0;
 z1 = X1[j1];
 z2 = X2[j2];
 if (N1 <= 1) goto seiuno;
 if (N2 <= 1) goto sesent;
  if (fabs(z2-z1) <= EP2) goto tredos;
  if ((z1-z2) == 0.0) goto tredos;
    if ((z1-z2) > 0.0) goto veinte;

    diez: if ((*J) < (M-1)) ++(*J);
	      X[(*J)] = (double)(z1);
	      Y[(*J)] = Y1[j1];
	      ++j1;
	      z1 = (double)(X1[j1]);
	      if (fabs(z2-z1) <= EP2) goto tredos;
	      if ((z1-z2) < 0.0) goto diez;
	      goto tredos;


  veinte:     ++j2;
	      z2 = (double)(X2[j2]);
	      if (fabs(z2-z1) <= EP2) goto tredos;
	      if ((z1-z2) > 0.0) goto veinte;
	      goto tredos;

  treint:     if (j1 >= N1) goto seiuno;
	      if (j2 >= N2) goto sesent;
	      z1 = (double)(X1[j1]);
	      z2 = (double)(X2[j2]);
  tredos:     z  = (z1 < z2) ? z1 : z2;
	      w1 = visli(z,X1,Y1, (j1 > 1) ? j1 : 1 );
	      w2 = visli(z,X2,Y2, (j2 > 1) ? j2 : 1 );
	      if (fabs(z1-z) <= EP2) ++j1;
	      if (fabs(z2-z) <= EP2) ++j2;
	      w = S * (((S*w1) > (S*w2)) ? (S*w1) : (S*w2));
	      d = w1-w2;

	      if (l || ((*J) <= 0)) goto cuados;
	      if ((fabs(d) > EP2) && (fabs(e) > EP2))
	    {
	      s1 = (d >= 0.0) ? 1.0 : -1.0;
	      s2 = (e >= 0.0) ? 1.0 : -1.0;
	      if (s1 != s2)
	      { if ((*J) < (M-1)) ++(*J);
		X[(*J)] = (double)(u - e*((z-u)/(d-e)));
		Y[(*J)] = (double)(ww + (X[(*J)]-u)*((w1-ww)/(z-u)));
	      };
	    }


	    if (vissl(z,w,X,Y,(*J)) > EP1)
  cuados:    if ((*J) < (M-1))  ++(*J);
	     X[(*J)] = (double)(z);
	     Y[(*J)] = (double)(w);
	     l  = 0;
	     e  = d;
	     u  = z;
	     ww = w1;
	  goto treint;

  sesent: if (j1 >= N1) goto seiuno;
	  if ((*J) < (M-1)) ++(*J);
	  X[(*J)] = X1[j1];
	  Y[(*J)] = Y1[j1];
	++j1;
	goto sesent;

  seiuno:  ++(*J);

}
