#include <math.h>
#include "vis.h"

#define SIZE 150
#define VF 0.333
#define M    1

static double  U[SIZE],V[SIZE];

void visrs(double Z1, double ZE[], double Z2, int NX, int MX,
	   int NY, int MY, double TH, void (*pl)())

/*       visrs.c
   [Rotated sequence]
   Secuencia rotada.

   (Z1,Z2)   Rango de ZE.
   ZE[]      Arreglo de los valores de la funcion ZE(J,I).
   NX,NY     Rango inicial de la matriz de J e I.
   MX,MY     Rango final de la matriz J e I.
   TH        Angulo de rotacion (en grados).
   pl        Funcion de conversion de coordenadas.
   (H.V.McIntosh, 19.DIC.74)
   (Fco. Gil, 23.DIC.84)
   (Conv. a C., G. Cisneros, 8.JUL.88)
   (Angel Canto, 16.Mar.89)
*/
{
 int L;         /* DIRECCION DE LA VISTA (1=OESTE, -1=ESTE).            */
/*     M           DIRECCION DE LA VISTA (1=SUR  , -1=NORTE).           */
 int n,mm,i0,j0,k0,i,j,k,ixji;
 double tt,si,co,el,em,zs,sf,u0,v0,dui,duj,dvi,dvj,eu,ve;


 if(TH < 0.0) L = -1;
 else L = 1;
 n = L * M;
 mm = 1;
 tt = TH * 0.0174533;
 si = sin(tt);
 co = cos(tt);
 el = (double)(L);
 em = (double)(M);
 zs = (1.0-VF) / (Z2-Z1);
 sf = 1.0 / (fabs(co) + fabs(si));
 u0 = 0.25 * (el+1.0) * (1.0-sf*(co-si));
 v0 = 0.15 * (el-1.0) * sf*si;
 dui = -(sf*si) / (double)(MY-1);
 duj = (sf*co) / (double)(MX-1);
 dvi = (VF*sf*co) / (double)(MY-1);
 dvj = (VF*sf*si) / (double)(MX-1);

 i0 = (M > 0) ? 2 : (NY-1);
 j0 = (L > 0) ? 1 : NX;
 k0 = (n > 0) ? -1 : SIZE;

 do{
    do{ k=k0;
	i = visbd(i0, NY+1, 0);
	j = j0;
	eu = u0+dui*(double)(i-1)+duj*(double)(j-1);
	ve = v0+dvi*(double)(i-1)+dvj*(double)(j-1);
	do{
	   if((i >= 1) && (i <= NY))
	     {
	      k    = visbd(k+n, SIZE-1, 0);
	      U[k] = (double)(eu);
	      ixji = (i-1) + (MY*(j-1));
	      V[k] = (double)(ve + zs*(ZE[ixji]-Z1));  };
	   i  -= M;
	   eu -= em * dui;
	   ve -= em * dvi;
	   if((i < 1) || (i > NY)) break;
	   k = visbd(k+n, SIZE-1, 0);
	   U[k] = (double)(eu);
	   ixji = (i-1) + (MY * (j-1));
	   V[k] = (double)(ve + zs*(ZE[ixji]-Z1));
	   j += L;
	   eu += el * duj;
	   ve += el * dvj;
	  } while ((j >= 1) && (j <= NX));

	if(L > 0) visho(&U[0], &V[0], k+1 ,   mm, pl);
	if(L < 0) visho(&U[k], &V[k], SIZE-k, mm, pl);
	mm = -mm;
	i0 += M;
      } while ((i0 >= 0) && (i0 <= NY+1));
	j0 += L;
   } while ((j0 >= 1) && (j0 <= NX));
}
