#include <math.h>

#include "vis.h"

void vissp(double *RHO, double *PHI, double R, double T, double P,
	   double O[])

/*       vissp.c
   [Spherical projection]
   Proyeccion esferica.
   Determina el palno en coordenadas polares en la pagina del graficador,
   de un punto de superficies esfericas.

   *RHO,*PHI   Plano de coordenadas polares
   R,T,P       Superficie en coordenadas polares.
   O[]         Matriz ortogonal (de rotacion)
   (H.V.McIntosh, 20.MAY.74)
   (Fco. Gil, 23.DIC.84)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16 Mar.89)
*/
{
  double x,y,z,u,v;

 x = R * sin(T) * cos(P);
 y = R * sin(T) * sin(P);
 z = R * cos(T);
 u = O[0]*x + O[1]*y + O[2]*z;
 v = O[3]*x + O[4]*y + O[5]*z;
 *RHO = (double)(sqrt(u*u + v*v));
 *PHI = (double)(atan2(v,u) / 6.28318);
}
