#include <math.h>

#include "vis.h"

#define SIZE 200

static  double  AZ[SIZE],RA[SIZE],OR[SIZE];
static  int     PE[SIZE];

void visss(double FU[], int J1, int J2, int NP, int I1, int I2,
	   int NT, int L, int M, double Q, int B, double S,
	   double O[], void (*pr)(), void (*pl)())

/*       visss.c
   [Spherical sequence]
   Secuencia esferica.

   FU[]   Arreglo de valores de la funcion.
   J1,J2  Intervalo de PHI.
   NP,NT  FU[NP][NT].
   I1,I2  Intervalo de THETA.
   L      L= (1= Vista Oeste, -1= Vista Este).
   M      M= (1= Vista Sur  , -1= Vista Norte).
   Q      Factor de rotacion.
   B      Bandera de rotacion positiva.
   S      S =  1.0 Grafica funcion positiva.
	  S = -1.0 Grafica funcion negativa.
	  S =  0.0 Grafica toda la funcion.
   O[]    Matriz ortogonal.
   pr     Funcion de proyeccion.
   pl     Funcion de conversion de coordenadas.
   (H.V.McIntosh, 15.MAY.75)
   (Fco. Gil, 4.ENE.85)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 23.Mar.89)
*/
{
  int t3;
  int nl,nm,n,il,iu,nn,i0,j0,k0,i,j,k,ixji,mm,k1,k2,m1,kmm1,jp,it;
  double el,em,dp,dt,tn,t,p,fv,tv,pf,ph,th;
  double fs,v1,v2,v3,t1,t2,t4;

  nl = sgn(L);
  nm = sgn(M);
  n = nl * nm;
  il = I1-abs(M);
  iu = I2+abs(M);
  nn = 1;
  el = (double)(L);
  em = (double)(M);
  dp = 6.28 / (double)(NP);
  dt = 3.14 / (double)(NT);

  visnp(&ph, &th, &jp, &it, NP, NT, O);

  tn = tan(th);
  i0 = (nm > 0) ? (I1+M) : (I2+M);
  j0 = (nl > 0) ? J1 : J2;
  k0 = (n  > 0) ? -1 : SIZE;

 do{
    do{
	k = k0;
	i = visbd(i0, iu, il);
	j = j0;
	t = th+dt*(double)(i-it);
	p = ph+dp*(double)(j-jp);

       do{
	  if((i >= I1) && (i <= I2))
	  { k = visbd(k+n, SIZE-1, 0);

	    ixji = (((NP+j-1) % NP) * NT) + i - 1;
	    fv = (double)(FU[ixji]);

	    (*pr)(&RA[k], &AZ[k], fabs(fv), t, p, O);

	    PE[k] = ((S*fv) >= 0.0);
	    fs = ((1.57079-t) >= 0) ? 1.0 : -1.0;
	    v1 = tan(t);
	    v2 = cos(p-ph);
	    v2 *= tn;
	    v1 -= v2;
	    OR[k] = (double)(Q * fs * v1) ;
	    if (OR[k] == -0.0)
	    OR[k] = (double)(0.00000023842);
	    if (OR[k] == 0.0)
	    OR[k] = (double)(-0.00000023842);
	   };

	  i -= M;
	  t -= em * dt;
	  if ((i > I2) || (i < I1)) break;
	  k = visbd(k+n, SIZE-1, 0);

	    ixji = (((NP+j-1) % NP) * NT) + i - 1;
	  fv = (double)(FU[ixji]);

	  (*pr)(&RA[k], &AZ[k], fabs(fv), t, p, O);

	  PE[k] = ((S*fv) >= 0.0);
	  fs = ((1.57079-t) >= 0) ? 1.0 : -1.0;
   v1 = tan(t);
   v2 = cos(p-ph);
   v2 *= tn;
   v1 -= v2;
	  OR[k] = (double)(Q * fs * v1) ;
	if (OR[k] == -0.0)
	  OR[k] = (double)(0.00000023842);
	  if (OR[k] == 0.0)
	  OR[k] = (double)(-0.00000023842);
	  j += L;
	  p += el * dp;
	 } while ((j <= J2) && (j >= J1));

	if ( n <= 0 )
	{  m1 = SIZE - k ;
	   for (mm = 0 ; mm < m1 ; mm++)
	   {
	     kmm1 = k+mm;

	     AZ[mm] = AZ[kmm1];
	     RA[mm] = RA[kmm1];
	     PE[mm] = PE[kmm1];
	     OR[mm] = OR[kmm1];  };
	  k = m1-1;  };

      if (k > 0)
      {
	if (!B)
	for (mm=0; mm <= k ; mm++)
	if (AZ[mm] < 0.0) AZ[mm] = AZ[mm]+1.0;

	if (AZ[0] > AZ[k])
	for (mm=0; mm <= k; mm++)
	{
	  t1 = AZ[mm];
	  t2 = RA[mm];
	  t3 = PE[mm];
	  t4 = OR[mm];
	  kmm1 = k-mm;

	  AZ[mm] = AZ[kmm1];
	  RA[mm] = RA[kmm1];
	  PE[mm] = PE[kmm1];
	  OR[mm] = OR[kmm1];
	  AZ[kmm1] = t1;
	  RA[kmm1] = t2;
	  PE[kmm1] = t3;
	  OR[kmm1] = t4;        };

	k1 = 0;

  while(k1 < k)
       {
	if(OR[k1] <= 0.0)
	{
	   k2 = k1;

	 if (k1 > 0)
	 {
	  --k1;
 AZ[k1] = (double)(AZ[k1] - OR[k1] * 
	  ((AZ[k1+1]-AZ[k1])/(OR[k1+1]-OR[k1])) );
 RA[k1] = (double)(RA[k1] - OR[k1] * 
	  ((RA[k1+1]-RA[k1])/(OR[k1+1]-OR[k1])) );
	 };

	 while((k2 <= k) && (OR[k2] <= 0.0))
	  ++k2;

	 if ((OR[k2] > 0.0) && (k2 <= k))
	  {
 AZ[k2] = (double)(AZ[k2] - OR[k2] * 
	  ((AZ[k2-1]-AZ[k2])/(OR[k2-1]-OR[k2])) );
 RA[k2] = (double)(RA[k2] - OR[k2] * 
	  ((RA[k2-1]-RA[k2])/(OR[k2-1]-OR[k2])) );
	  ++k2;
	  };
	  if ((k2-k1) > 1)
	  {
	  if(AZ[k1] >= AZ[k1+1]) AZ[k1] = (double)(AZ[k1+1]-0.0025);
	  if(AZ[k2-1] <= AZ[k2-2]) AZ[k2-1] = (double)(AZ[k2-2]+0.0025);
	  };
	 if ((k2-k1) > 1) visch(&AZ[k1], &RA[k1], &PE[k1], k2-k1, nn, pl);
	 k1 = k2;
	};
       ++k1;
       };
      };
	nn = -nn;
	i0 += M;
      } while ((i0 >= (I1-1)) && (i0 <= (I2+1)));
	j0 += L;
   } while ((j0 >= J1) && (j0 <= J2));
}
