#include <math.h>

#include "vis.h"

#define SIZE 100
#define SIZE2 200
#define SIZE3 300
#define M     1

static  double  G1[SIZE3],H1[SIZE3];
static  double  X1[SIZE3],B1[SIZE3],T1[SIZE3];
static  double  U1[SIZE3],F1[SIZE3];
static  double  U[SIZE],V1[SIZE],V2[SIZE],V3[SIZE];

void vistr(double Z1, double S1[], double S2[], double S3[], 
	   double Z2, int NX, int MX, int NY, int MY, double US, 
	   double VS, double VD, int L, int IS, void (*pl)())

/*       vistr.c
   [Triple surface]
   Superficie triple.

   (Z1,Z2)           Rango de los valores de las superficies.
   S1[],S2[],S3[]    Arreglo de datos de tres superficies.
   NX,MX             Dimensionamiento comun de los arreglos 
		     S1,S2,S3.[MX,MY].
   NY,MY             Seccion usada en S1,S2,S3.[NX,NY]
   US,VS             Total direccion en U,V.
   VD                Direccionamiento vertical en el segmento de la funcion.
   L                 = 1 Oeste , = -1 Este.
   IS                Separadas las superficies :
		     = 1 => SI  , = -1 => NO.
   pl                Funcion de conversion de coordenadas.
   (H.V.McIntosh, 16.MAY.75)
   (Fco. Gil, 16.ENE.85)
   (Conv. a C, G. Cisneros, 8.JUL.88)
   (Angel Canto, 16.Mar.89)
*/
{
  int    n1,n2,n3,sequ,n,i0,j0,k0,i,j,k;
  int    ixji,nk,kk,ii,ng1,ng2,ng3,nu1,nu2,nu3;
  double dd1,dd2,ef,el,em,zs,te,dui,duj,dvi,eu,ve;

 n1 = 0;
 n2 = 0;
 n3 = 0;
 sequ = 1;
 n = L * M;
 dd1  = (VD+VS);
 dd2  = 2.0 * dd1;
 ef   = 1.0 - dd2;
 el   = (double)(L);
 em   = (double)(M);
 zs   = ef / (Z2-Z1);
 te   = 0.5 * (el+1.0);
 dui  = -(el*US) / (double)(MY-1);
 duj  = (1.0-US) / (double)(MX-1);
 dvi  = VS / (double)(MY-1);

 i0 = (M > 0) ? 2 : (NY-1);
 j0 = (L > 0) ? 1 : NX;
 k0 = (n > 0) ? -1 : SIZE;

 do{
    do{
	k  = k0;
	i  = visbd(i0, NY+1, 0);
	j  = j0;
	eu = te*US+dui*(double)(i-1)+duj*(double)(j-1);
	ve =       dvi*(double)(i-1);

       do{
	   if ((i >= 1) && (i <= NY))
	   {
	     k = visbd(k+n, SIZE-1, 0);
	     U[k] = (double)(eu);
	     ixji = (i-1) + (MY*(j-1));
	     V1[k] = (double)(ve + zs*(S1[ixji]-Z1));
	     V2[k] = (double)(ve + zs*(S2[ixji]-Z1));
	     V3[k] = (double)(ve + zs*(S3[ixji]-Z1));
	   };
	   i -= M;
	   eu -= dui;
	   ve -= em * dvi;
	   if ((i < 1) || (i > NY)) break;
	   k = visbd(k+n, SIZE-1, 0);
	   U[k] = (double)(eu);
	   ixji = (i-1) + (MY*(j-1));
	   V1[k] = (double)(ve + zs*(S1[ixji]-Z1));
	   V2[k] = (double)(ve + zs*(S2[ixji]-Z1));
	   V3[k] = (double)(ve + zs*(S3[ixji]-Z1));
	   j += L;
	   eu += el * duj;
	 } while ((j >=1) && (j <= NX));

	if (L <= 0)
	{
	  nk = SIZE - k-1;
	  for (kk = 0 ; kk <= nk ; kk++)
	  {
	    ii = k + kk;
	    U[kk] = U[ii];
	    V1[kk] = V1[ii];
	    V2[kk] = V2[ii];
	    V3[kk] = V3[ii];    };
	  k = nk;       };
	visrb(G1, H1, &ng1, SIZE, U, V1, k+1, U, V2, k+1, 1.0);
	visrb(&U1[SIZE2], &F1[SIZE2], &nu3, SIZE, G1, H1, ng1, U, V3,
	      k+1, 1.0);
	visrb(G1, H1, &ng1, SIZE, U, V2, k+1, U, V3, k+1, -1.0);
	visrb(&G1[SIZE], &H1[SIZE], &ng2, SIZE, U, V3, k+1, U, 
	      V1, k+1, -1.0);
	visrb(&G1[SIZE2], &H1[SIZE2], &ng3, SIZE, U, V1, k+1, U, V2, 
	      k+1, -1.0);
	visrb(U1, F1, &nu1, SIZE, G1, H1, ng1, &G1[SIZE], &H1[SIZE], 
	      ng2, 1.0);
	visrb(&U1[SIZE], &F1[SIZE], &nu2, SIZE, U1, F1, nu1, &G1[SIZE2],
	      &H1[SIZE2], ng3, 1.0);
	visrb(U1, F1, &nu1, SIZE, &G1[SIZE2], &H1[SIZE2], ng3, U, V3, 
	      k+1, -1.0);

	if (IS >= 0)
	{
	  for (kk = 0 ; kk < nu1 ; kk++)
	   F1[kk] = (double)(ef*F1[kk]);
	  for (kk = 0 ; kk < nu2 ; kk++)
	   F1[kk+SIZE] = (double)(ef*F1[kk+SIZE]+dd1);
	  for (kk = 0 ; kk < nu3 ; kk++)
	   F1[kk+SIZE2] = (double)(ef*F1[kk+SIZE2]+dd2);        };

if (!sequ)
{
visrb(G1, H1, &ng1, SIZE, &U1[SIZE2], &F1[SIZE2], nu3, &X1[SIZE],
      &B1[SIZE], n2, 1.0);
vishh(&X1[SIZE2], &B1[SIZE2], &T1[SIZE2], &n3, G1, H1, 
      ng1, -1, pl);
visrb(G1, H1, &ng1, SIZE, &U1[SIZE], &F1[SIZE], nu2, X1, B1, n1, 1.0);
visrb(&G1[SIZE], &H1[SIZE], &ng2, SIZE, G1, H1, ng1, &X1[SIZE2],
      &T1[SIZE2], n3, -1.0);
vishh(&X1[SIZE], &B1[SIZE], &T1[SIZE], &n2, &G1[SIZE], &H1[SIZE], 
      ng2,  1, pl);
visrb(G1, H1, &ng1, SIZE, U1, F1, nu1, &X1[SIZE], &T1[SIZE], n2, -1.0);
vishh(X1, B1, T1, &n1, G1, H1, ng1, -1, pl);
	sequ = !sequ;
}
else
{
visrb(G1, H1, &ng1, SIZE, U1, F1, nu1, &X1[SIZE], &T1[SIZE], n2, -1.0);
vishh(X1, B1, T1, &n1, G1, H1, ng1,  1, pl);
visrb(G1, H1, &ng1, SIZE, &U1[SIZE], &F1[SIZE], nu2, X1, B1, n1, 1.0);
visrb(&G1[SIZE], &H1[SIZE], &ng2, SIZE, G1, H1, ng1, &X1[SIZE2],
      &T1[SIZE2], n3, -1.0);
vishh(&X1[SIZE], &B1[SIZE], &T1[SIZE], &n2, &G1[SIZE], &H1[SIZE], 
      ng2, -1, pl);
visrb(G1, H1, &ng1, SIZE, &U1[SIZE2], &F1[SIZE2], nu3, &X1[SIZE],
      &B1[SIZE], n2, 1.0);
vishh(&X1[SIZE2], &B1[SIZE2], &T1[SIZE2], &n3, G1, H1, ng1,  1, pl);
	sequ = !sequ;
};
      i0 += M;
      } while ((i0 >= 0) && (i0 <= NY+1));
	j0 += L;
   } while ((j0 >= 1) && (j0 <= NX));
}
