extern int NN0,M;

#include <math.h>

#include "vis.h"

#define SIZE 200
#define UP 0

#define EP 1.0E-3
#define EP1 1.0E-5
#define EP2 0.5E-4

void visbo(double X1[], double T1[], double B1[], double X0[], 
	   double T0[], double B0[], double X[], double Y[], 
	   int P[], int N, int I, void (*pl)())

/*       vsb.c
   [Bounds]
   Cotas.

   X1[],T1[],B1[],X0[],T0[],B0[]   Cotas.
   X[]     X[N] Arreglo de argumentos.
   Y[]     Y[N] Arreglo de valores de la funcion.
   P[]     P[N] Arreglo de banderas de visibildad.
   N       Longitud de arreglos.
   I       Direccion de movimiento de pluma (1=delante, -1=atras).
   pl      Funcion de conversion de coordenadas.
   (H.V.McIntosh, 10.MAY.75)
   (Fco. Gil, 23.DIC.83)
   (Conv. a C, G. Cisneros, 30.JUN.88)
   (Angel Canto, 16 Mar.89)
*/
{
  int i,j,k,l,jj,kk,vv,j0,j1;
  double s,et,eb,z,z0,ex,wy,to,bo,te,be;
  double dt,db,vt,vb,f,yi,xi,xx,yy,tt,bb;
  double s1,s2;
  double U[2];

 if (N <= 1) return;

/* Inicializacion.         */

 for (i=0; i < N; i++)
  { if (X[i] == -0.0) X[i] = 0.0;
    if (Y[i] == -0.0) Y[i] = 0.0;};

 xx = yy = tt = bb = 0.0;
 vv = 0;
 j  = (I > 0) ? 0 : N-1;
 j1 = (I > 0) ? -1 : M;
 (*pl)((double)X[j], (double)Y[j], UP);

 if (NN0 <= 1) goto sesuno;

  s = (double)(I);
 et = 1.0;
 eb = -1.0;
  l = 1;
  k = (I < 0) ? 1 : 0;
 j0 = (I > 0) ? 0 : NN0-1;
  z = X[j];
 z0 = X0[j0];

 if (( fabs(z-z0) <= EP2) || (s*(z-z0) == 0.0)) goto tredos;

/* Si la funcion esta definida mientras las cotas no estan, la funcion
   esta visible y puede ser copiada, estableciendo nuevas cotas. */

 if (s*(z-z0) <= 0.0)
  { do {  j1 = visbd(I+j1, M-1, 0);
	  (*pl)((double)X[j], (double)Y[j], P[j]);
	  X1[j1] = X[j];
	  T1[j1] = Y[j];
	  B1[j1] = Y[j];
	  j += I;
	 if ( fabs(X[j] - z0) <= EP2) break;
       } while (s*(X[j]-z0) < 0.0); }
 else

/* Si las cotas, pero no la funcion, estan definidas, estas persisten. */

  { do { j1 = visbd(I+j1, M-1, 0);
	X1[j1] = X0[j0];
	T1[j1] = T0[j0];
	B1[j1] = B0[j0];
	j0 += I;
	if ( fabs(z-X0[j0]) <= EP2) break;
      } while (s*(z-X0[j0]) > 0.0); };

/* En el punto donde ambos la funcion o la cota estan definidas, puede
   ser necesario obtener el otro por interpolacion lineal, al menos que
   los puntos de definicion coincidan.  */

treint: if((j < 0) || (j >= N)) goto cincue;
	if((j0 < 0) || (j0 >= NN0)) goto sesent;
	z = X[j];
	z0 = X0[j0];

tredos: ex = s * (((s*z) < (s*z0)) ? (s*z) : (s*z0));
	wy = visli(ex, X, Y, visbd(j +k, N-1, 1) );
	to = visli(ex, X0, T0, visbd(j0 +k, NN0-1, 1) );
	bo = visli(ex, X0, B0, visbd(j0 +k, NN0-1, 1) );
	if ( fabs(ex-z0) <= (EP2)) j0 += I;
	if ( fabs(ex-z) <= (EP2)) j += I;

/* Posible interseccion entre la funcion y la cota, debe ser grabada
   asi como descrita la nueva cota correctamente.
   Es necesario tener cuidado para eludir intersecciones triviales,
    puede ocurrir un punto final. */

	te = (wy > to) ? wy : to;
	be = (wy < bo) ? wy : bo;
	dt = wy - to;
	db = wy - bo;
	vt = et + dt;
	vb = eb + db;
	if (l) goto cuasei;

	jj = -1;
s1 = (dt >= 0.0) ? 1.0 : -1.0;
s2 = (et >= 0.0) ? 1.0 : -1.0;
	if( s1 != s2 )
	{  vt = dt-et;
	   ++jj;
	   U[jj] = xx-et*((ex-xx)/(dt-et)); };

s1 = (db >= 0.0) ? 1.0 : -1.0;
s2 = (eb >= 0.0) ? 1.0 : -1.0;
	if( s1 != s2 )
	{  vb = db-eb;
	   ++jj;
	   U[jj] = xx-eb*((ex-xx)/(db-eb)); };

	if(jj != -1)
	for (kk=0 ; kk <= jj ; kk++)
	{
	  if((kk == 0) && (jj == 0)) xi = U[0];
	  if((kk == 0) && (jj == 1)) 
	     xi = s*((s*U[0] < s*U[1]) ? s*U[0] : s*U[1]);
	  if(kk == 1)                
	     xi = s*((s*U[0] > s*U[1]) ? s*U[0] : s*U[1]);
	   f = (xi-xx)/(ex-xx);
	  yi = yy+f*(wy-yy);
	   i = (kk == 0) && vv;
	  (*pl)(xi, yi, i);
	  if(( fabs(xx-xi) > EP2) && ( fabs(xi-ex) > EP2))
	   { if((kk != 1) || ( fabs(U[0]-U[1]) > EP2))
	      {
		j1 = visbd(I+j1, M-1, 0);
		X1[j1] = xi;
		T1[j1] = tt+f*(to-tt);
		B1[j1] = bb+f*(bo-bb);
 };};  };

	if((j1 < 1) || (j1 > M-2)) goto cuasei;
	if(vissl(ex, te, X1, T1, j1+k) > EP1) goto cuasei;
	if(vissl(ex, be, X1, B1, j1+k) <= EP1) goto cuaoch;

cuasei: j1 = visbd(I+j1, M-1, 0);

cuaoch: X1[j1] = ex;
	T1[j1] = te;
	B1[j1] = be; vv = (j<0) ? P[0]:P[j];
	vv &= ((vt > EP) || (-vb > EP));
	(*pl)(ex, wy, vv);
	l = 0;
	et = dt;
	eb = db;
	xx = ex;
	yy = wy;
	tt = to;
	bb = bo;
	goto treint;

/* Terminacion.
   Si la funcion se agota antes que las cotas, copia estas. */

cincue: if((j0 < 0) || (j0 >= NN0)) goto setent;
	j1 = visbd(I+j1, M-1, 0);
	X1[j1] = X0[j0];
	T1[j1] = T0[j0];
	B1[j1] = B0[j0];
	j0 += I;
	goto cincue;

/* Si las cotas son agotadas antes que la funcion, copia la parte
   restante de la funcion, la cual esta visible. */

sesent:  (*pl)(ex, wy, UP);

sesuno: if((j < 0) || (j >= N)) goto setent;
	(*pl)((double)X[j], (double)Y[j], P[j]);
	j1 = visbd(I+j1, M-1, 0);
	X1[j1] = X[j];
	T1[j1] = Y[j];
	B1[j1] = Y[j];
	j += I;
	goto sesuno;

/* Copia las nuevas cotas sobre la viejas, recorriendo cuando es
   necesario. */

setent: NN0 = ((M+1)*(1-I))/2+I*(j1+1);
	j1 = ((j1+2-I*(j1))/2)-1;
	for (j0=0 ; j0 < NN0 ; j0++)
	{
	  X0[j0] = X1[j1];
	  T0[j0] = T1[j1];
	  B0[j0] = B1[j1];
	  ++j1;
	}
}
